@@ -1,27 +1,23 @@
 # -*- coding: utf-8 -*-
-"""Random Interval Spectral Forest (RISE).
+"""Random Interval Spectral Ensemble (RISE)."""
 
-Implementation of Deng's Time Series Forest, with minor changes.
-"""
+__author__ = ["TonyBagnall"]
+__all__ = [
+    "RandomIntervalSpectralEnsemble",
+    "acf",
+    "matrix_acf",
+    "ps",
+]
 
-__author__ = ["Tony Bagnall", "Yi-Xuan Xu"]
-__all__ = ["RandomIntervalSpectralForest", "acf", "matrix_acf", "ps"]
-
-
-from numba import int64, prange, jit
 import numpy as np
-from joblib import Parallel
-from joblib import delayed
+from joblib import Parallel, delayed
+from numba import int64, jit, prange
 from sklearn.base import clone
-from sklearn.ensemble._forest import ForestClassifier
+from sklearn.ensemble._base import _partition_estimators
 from sklearn.tree import DecisionTreeClassifier
-from sklearn.utils.multiclass import class_distribution
 from sklearn.utils.validation import check_random_state
 
 from sktime.classification.base import BaseClassifier
-from sklearn.ensemble._base import _partition_estimators
-from sktime.utils.validation.panel import check_X
-from sktime.utils.validation.panel import check_X_y
 
 
 def _transform(X, interval, lag):
@@ -128,31 +124,30 @@ def _produce_intervals(
     return intervals
 
 
-class RandomIntervalSpectralForest(ForestClassifier, BaseClassifier):
-    """Random Interval Spectral Forest (RISE).
+class RandomIntervalSpectralEnsemble(BaseClassifier):
+    """Random Interval Spectral Ensemble (RISE).
 
     Input: n series length m
-    for each tree
-        sample a random intervals
-        take the ACF and PS over this interval, and concatenate features
-        build tree on new features
-    ensemble the trees through averaging probabilities.
+    For each tree
+        - sample a random intervals
+        - take the ACF and PS over this interval, and concatenate features
+        - build tree on new features
+    Ensemble the trees through averaging probabilities.
 
     Parameters
     ----------
-    n_estimators : int, optional (default=200)
+    n_estimators : int, default=200
         The number of trees in the forest.
-    min_interval : int, optional (default=16)
+    min_interval : int, default=16
         The minimum width of an interval.
-    acf_lag : int, optional (default=100)
+    acf_lag : int, default=100
         The maximum number of autocorrelation terms to use.
-    acf_min_values : int, optional (default=4)
+    acf_min_values : int, default=4
         Never use fewer than this number of terms to find a correlation.
-    n_jobs : int or None, optional (default=None)
+    n_jobs : int, default=1
         The number of jobs to run in parallel for both `fit` and `predict`.
-        ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
         ``-1`` means using all processors.
-    random_state : int, RandomState instance or None, optional (default=None)
+    random_state : int, RandomState instance or None, default=None
         If int, random_state is the seed used by the random number generator;
         If RandomState instance, random_state is the random number generator;
         If None, the random number generator is the RandomState instance used
@@ -160,35 +155,41 @@ class RandomIntervalSpectralForest(ForestClassifier, BaseClassifier):
 
     Attributes
     ----------
-    n_classes : int
-        The number of classes, extracted from the data.
-    n_estimators : array of shape = [n_estimators] of DecisionTree classifiers
-    intervals : array of shape = [n_estimators][2]
+    n_classes_ : int
+        The number of classes.
+    classes_ : list
+        The classes labels.
+    intervals_ : array of shape = [n_estimators][2]
         Stores indexes of start and end points for all classifiers.
 
     Notes
     -----
-    ..[1] Jason Lines, Sarah Taylor and Anthony Bagnall, "Time Series Classification
-    with HIVE-COTE: The Hierarchical Vote Collective of Transformation-Based Ensembles",
-      ACM Transactions on Knowledge and Data Engineering, 12(5): 2018
-    https://dl.acm.org/doi/10.1145/3182382
-    Java implementation
-    https://github.com/uea-machine-learning/tsml/blob/master/src/main/java/tsml/
-    classifiers/frequency_based/RISE.java
+    For the Java version, see
+    `TSML <https://github.com/uea-machine-learning/tsml/blob/master/src/main/java/tsml/
+    classifiers/interval_based/RISE.java>`_.
+
+    References
+    ----------
+    .. [1] Jason Lines, Sarah Taylor and Anthony Bagnall, "Time Series Classification
+       with HIVE-COTE: The Hierarchical Vote Collective of Transformation-Based
+       Ensembles", ACM Transactions on Knowledge and Data Engineering, 12(5): 2018
+
+    Examples
+    --------
+    >>> from sktime.classification.interval_based import RandomIntervalSpectralEnsemble
+    >>> from sktime.datasets import load_unit_test
+    >>> X_train, y_train = load_unit_test(split="train", return_X_y=True)
+    >>> X_test, y_test = load_unit_test(split="test", return_X_y=True)
+    >>> clf = RandomIntervalSpectralEnsemble(n_estimators=10)
+    >>> clf.fit(X_train, y_train)
+    RandomIntervalSpectralEnsemble(...)
+    >>> y_pred = clf.predict(X_test)
     """
 
-    # Capability tags
-    capabilities = {
-        "multivariate": False,
-        "unequal_length": False,
-        "missing_values": False,
-        "train_estimate": False,
-        "contractable": False,
+    _tags = {
+        "capability:multithreading": True,
     }
 
-    # TO DO: handle missing values, unequal length series and multivariate
-    # problems
-
     def __init__(
         self,
         n_estimators=500,
@@ -196,13 +197,9 @@ class RandomIntervalSpectralForest(ForestClassifier, BaseClassifier):
         min_interval=16,
         acf_lag=100,
         acf_min_values=4,
-        n_jobs=None,
+        n_jobs=1,
         random_state=None,
     ):
-        super(RandomIntervalSpectralForest, self).__init__(
-            base_estimator=DecisionTreeClassifier(random_state=random_state),
-            n_estimators=n_estimators,
-        )
         self.n_estimators = n_estimators
         self.max_interval = max_interval
         self.min_interval = min_interval
@@ -211,8 +208,11 @@ class RandomIntervalSpectralForest(ForestClassifier, BaseClassifier):
         self.n_jobs = n_jobs
         self.random_state = random_state
 
-        # We need to add is-fitted state when inheriting from scikit-learn
-        self._is_fitted = False
+        self.intervals_ = []
+
+        self.base_estimator = DecisionTreeClassifier(random_state=random_state)
+
+        super(RandomIntervalSpectralEnsemble, self).__init__()
 
     @property
     def feature_importances_(self):
@@ -222,7 +222,7 @@ class RandomIntervalSpectralForest(ForestClassifier, BaseClassifier):
             "RandomIntervalSpectralForest is currently not supported."
         )
 
-    def fit(self, X, y):
+    def _fit(self, X, y):
         """Build a forest of trees from the training set (X, y).
 
         using random intervals and spectral features.
@@ -241,9 +241,7 @@ class RandomIntervalSpectralForest(ForestClassifier, BaseClassifier):
         -------
         self : object
         """
-        X, y = check_X_y(X, y, enforce_univariate=True, coerce_to_numpy=True)
         X = X.squeeze(1)
-
         n_instances, self.series_length = X.shape
         self.min_interval_, self.max_interval_ = self.min_interval, self.max_interval
         if self.max_interval_ not in range(1, self.series_length):
@@ -254,8 +252,6 @@ class RandomIntervalSpectralForest(ForestClassifier, BaseClassifier):
         rng = check_random_state(self.random_state)
 
         self.estimators_ = []
-        self.n_classes = np.unique(y).shape[0]
-        self.classes_ = class_distribution(np.asarray(y).reshape(-1, 1))[0][0]
         # self.intervals = _produce_intervals(
         #     self.n_estimators,
         #     self.min_interval,
@@ -263,8 +259,8 @@ class RandomIntervalSpectralForest(ForestClassifier, BaseClassifier):
         #     self.series_length,
         #     rng
         # )
-        self.intervals = np.empty((self.n_estimators, 2), dtype=int)
-        self.intervals[:] = [
+        self.intervals_ = np.empty((self.n_estimators, 2), dtype=int)
+        self.intervals_[:] = [
             _select_interval(
                 self.min_interval_, self.max_interval_, self.series_length, rng
             )
@@ -287,12 +283,12 @@ class RandomIntervalSpectralForest(ForestClassifier, BaseClassifier):
         ]
 
         # Parallel loop
-        worker_rets = Parallel(n_jobs=self.n_jobs)(
+        worker_rets = Parallel(n_jobs=self._threads_to_use)(
             delayed(_parallel_build_trees)(
                 X,
                 y,
                 tree,
-                self.intervals[i],
+                self.intervals_[i],
                 self.acf_lag_,
                 self.acf_min_values,
             )
@@ -304,10 +300,9 @@ class RandomIntervalSpectralForest(ForestClassifier, BaseClassifier):
             self.lags[i] = lag
             self.estimators_.append(tree)
 
-        self._is_fitted = True
         return self
 
-    def predict(self, X):
+    def _predict(self, X):
         """Find predictions for all cases in X.
 
         Built on top of `predict_proba`.
@@ -324,10 +319,10 @@ class RandomIntervalSpectralForest(ForestClassifier, BaseClassifier):
         y : array of shape = [n_instances]
             The predicted classes.
         """
-        proba = self.predict_proba(X)
+        proba = self._predict_proba(X)
         return np.asarray([self.classes_[np.argmax(prob)] for prob in proba])
 
-    def predict_proba(self, X):
+    def _predict_proba(self, X):
         """Find probability estimates for each class for all cases in X.
 
         Parameters
@@ -350,27 +345,24 @@ class RandomIntervalSpectralForest(ForestClassifier, BaseClassifier):
         output : array of shape = [n_instances, n_classes]
             The class probabilities of all cases.
         """
-        # Check data
-        self.check_is_fitted()
-        X = check_X(X, enforce_univariate=True, coerce_to_numpy=True)
         X = X.squeeze(1)
 
         n_instances, n_columns = X.shape
         if n_columns != self.series_length:
-            raise TypeError(
+            raise ValueError(
                 "ERROR number of attributes in the train does not match "
                 "that in the test data."
             )
 
         # Assign chunk of trees to jobs
-        n_jobs, _, _ = _partition_estimators(self.n_estimators, self.n_jobs)
+        n_jobs, _, _ = _partition_estimators(self.n_estimators, self._threads_to_use)
 
         # Parallel loop
         all_proba = Parallel(n_jobs=n_jobs)(
             delayed(_predict_proba_for_estimator)(
                 X,
                 self.estimators_[i],
-                self.intervals[i],
+                self.intervals_[i],
                 self.lags[i],
             )
             for i in range(self.n_estimators)
