@@ -201,7 +201,7 @@ class _NaiveForecaster(_BaseWindowForecaster):
 
         # reshape last window, one column per season
         last_window = last_window.reshape(
-            np.int(np.ceil(self.window_length_ / self.sp_)), self.sp_
+            int(np.ceil(self.window_length_ / self.sp_)), self.sp_
         )
 
         return last_window
@@ -224,7 +224,7 @@ class _NaiveForecaster(_BaseWindowForecaster):
         # assume fh is sorted, i.e. max(fh) == fh[-1]
         # only slicing all the last seasons into last_window
         if fh[-1] > self.sp_:
-            reps = np.int(np.ceil(fh[-1] / self.sp_))
+            reps = int(np.ceil(fh[-1] / self.sp_))
             y_pred = np.tile(y_pred, reps=reps)
 
         # get zero-based index by subtracting the minimum
@@ -366,3 +366,25 @@ class NaiveForecaster(BaseForecaster):
         self : reference to self
         """
         return self._forecaster.update(y=y, X=X, update_params=update_params)
+
+    @classmethod
+    def get_test_params(cls):
+        """Return testing parameter settings for the estimator.
+
+        Returns
+        -------
+        params : dict or list of dict, default = {}
+            Parameters to create testing instances of the class
+            Each dict are parameters to construct an "interesting" test instance, i.e.,
+            `MyClass(**params)` or `MyClass(**params[i])` creates a valid test instance.
+            `create_test_instance` uses the first (or only) dictionary in `params`
+        """
+        params_list = [
+            {},
+            {"sp": 2},
+            {"strategy": "mean"},
+            {"strategy": "drift"},
+            {"strategy": "mean", "window_length": 5},
+        ]
+
+        return params_list
